@@ -25,6 +25,7 @@ import com.google.common.collect.Lists;
 import com.google.common.io.Files;
 import org.apache.flume.Context;
 import org.apache.flume.Event;
+import org.apache.flume.instrumentation.SourceCounter;
 import org.apache.flume.serialization.LineDeserializer;
 import org.apache.flume.source.SpoolDirectorySourceConfigurationConstants;
 import org.junit.After;
@@ -71,6 +72,7 @@ public class TestSpoolingFileLineReader {
           .spoolDirectory(tmpDir)
           .completedSuffix(completedSuffix)
           .deserializerContext(ctx)
+          .sourceCounter(new SourceCounter("dummy"))
           .build();
     } catch (IOException ioe) {
       throw Throwables.propagate(ioe);
@@ -420,7 +422,10 @@ public class TestSpoolingFileLineReader {
                 "file1line5\nfile1line6\nfile1line7\nfile1line8\n",
                 f2, Charsets.UTF_8);
 
-    // Expect to skip over first file
+    // Skip over first file, which is empty, and will return an empty event.
+    Event event = parser.readEvent();
+    assertEquals(0, event.getBody().length);
+
     List<String> out = bodiesAsStrings(parser.readEvents(8));
 
     parser.commit();
