@@ -20,16 +20,20 @@
 package org.apache.flume.source;
 
 import java.io.IOException;
-import java.net.Inet4Address;
 import java.net.InetAddress;
 import java.net.InetSocketAddress;
+import java.net.ServerSocket;
 import java.net.UnknownHostException;
 import java.nio.ByteBuffer;
 import java.security.cert.X509Certificate;
+import java.nio.channels.ServerSocketChannel;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.HashMap;
 import java.util.List;
 import java.util.concurrent.Executors;
+import java.util.function.Consumer;
+
 import javax.net.ssl.SSLContext;
 import javax.net.ssl.SSLEngine;
 import javax.net.ssl.TrustManager;
@@ -38,6 +42,7 @@ import javax.net.ssl.X509TrustManager;
 import org.apache.avro.ipc.NettyTransceiver;
 import org.apache.avro.ipc.specific.SpecificRequestor;
 import org.apache.flume.Channel;
+import org.apache.flume.ChannelException;
 import org.apache.flume.ChannelSelector;
 import org.apache.flume.Context;
 import org.apache.flume.Event;
@@ -47,12 +52,12 @@ import org.apache.flume.channel.ChannelProcessor;
 import org.apache.flume.channel.MemoryChannel;
 import org.apache.flume.channel.ReplicatingChannelSelector;
 import org.apache.flume.conf.Configurables;
+import org.apache.flume.instrumentation.SourceCounter;
 import org.apache.flume.lifecycle.LifecycleController;
 import org.apache.flume.lifecycle.LifecycleState;
 import org.apache.flume.source.avro.AvroFlumeEvent;
 import org.apache.flume.source.avro.AvroSourceProtocol;
 import org.apache.flume.source.avro.Status;
-import org.jboss.netty.channel.ChannelException;
 import org.jboss.netty.channel.ChannelPipeline;
 import org.jboss.netty.channel.socket.SocketChannel;
 import org.jboss.netty.channel.socket.nio.NioClientSocketChannelFactory;
@@ -62,9 +67,15 @@ import org.jboss.netty.handler.ssl.SslHandler;
 import org.junit.Assert;
 import org.junit.Before;
 import org.junit.Test;
+import org.mockito.Mockito;
+import org.mockito.internal.util.reflection.Whitebox;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import static org.mockito.Matchers.any;
+import static org.mockito.Matchers.anyListOf;
+import static org.mockito.Mockito.doThrow;
+
 public class TestAvroSource {
 
   private static final Logger logger = LoggerFactory
@@ -93,27 +104,16 @@ public class TestAvroSource {
   }
 
   @Test
-  public void testLifecycle() throws InterruptedException {
-    boolean bound = false;
+  public void testLifecycle() throws InterruptedException, IOException {
 
-    for (int i = 0; i < 100 && !bound; i++) {
-      try {
-        Context context = new Context();
+    Context context = new Context();
 
-        context.put("port", String.valueOf(selectedPort = 41414 + i));
-        context.put("bind", "0.0.0.0");
+    context.put("port", String.valueOf(selectedPort = getFreePort()));
+    context.put("bind", "0.0.0.0");
 
-        Configurables.configure(source, context);
+    Configurables.configure(source, context);
 
-        source.start();
-        bound = true;
-      } catch (ChannelException e) {
-        /*
-         * NB: This assume we're using the Netty server under the hood and the
-         * failure is to bind. Yucky.
-         */
-      }
-    }
+    source.start();
 
     Assert
         .assertTrue("Reached start or error", LifecycleController.waitForOneOf(
@@ -128,6 +128,62 @@ public class TestAvroSource {
         source.getLifecycleState());
   }
 
+  @Test
+  public void testSourceStoppedOnFlumeExceptionIfPortUsed()
+      throws InterruptedException, IOException {
+    final String loopbackIPv4 = "127.0.0.1";
+    final int port = 10500;
+
+    // create a dummy socket bound to a known port.
+    try (ServerSocketChannel dummyServerSocket = ServerSocketChannel.open()) {
+      dummyServerSocket.socket().setReuseAddress(true);
+      dummyServerSocket.socket().bind(new InetSocketAddress(loopbackIPv4, port));
+
+      Context context = new Context();
+      context.put("port", String.valueOf(port));
+      context.put("bind", loopbackIPv4);
+      Configurables.configure(source, context);
+      try {
+        source.start();
+        Assert.fail("Expected an exception during startup caused by binding on a used port");
+      } catch (FlumeException e) {
+        logger.info("Received an expected exception.", e);
+        Assert.assertTrue("Expected a server socket setup related root cause",
+            e.getMessage().contains("server socket"));
+      }
+    }
+    // As port is already in use, an exception is thrown and the source is stopped
+    // cleaning up the opened sockets during source.start().
+    Assert.assertEquals("Server is stopped", LifecycleState.STOP,
+            source.getLifecycleState());
+  }
+
+  @Test
+  public void testInvalidAddress()
+      throws InterruptedException, IOException {
+    final String invalidHost = "invalid.host";
+    final int port = 10501;
+
+    Context context = new Context();
+    context.put("port", String.valueOf(port));
+    context.put("bind", invalidHost);
+    Configurables.configure(source, context);
+
+    try {
+      source.start();
+      Assert.fail("Expected an exception during startup caused by binding on a invalid host");
+    } catch (FlumeException e) {
+      logger.info("Received an expected exception.", e);
+      Assert.assertTrue("Expected a server socket setup related root cause",
+          e.getMessage().contains("server socket"));
+    }
+
+    // As port is already in use, an exception is thrown and the source is stopped
+    // cleaning up the opened sockets during source.start().
+    Assert.assertEquals("Server is stopped", LifecycleState.STOP,
+        source.getLifecycleState());
+  }
+
   @Test
   public void testRequestWithNoCompression() throws InterruptedException, IOException {
 
@@ -135,68 +191,62 @@ public class TestAvroSource {
   }
 
   @Test
-  public void testRequestWithCompressionOnClientAndServerOnLevel0() throws InterruptedException, IOException {
+  public void testRequestWithCompressionOnClientAndServerOnLevel0()
+      throws InterruptedException, IOException {
 
     doRequest(true, true, 0);
   }
 
   @Test
-  public void testRequestWithCompressionOnClientAndServerOnLevel1() throws InterruptedException, IOException {
+  public void testRequestWithCompressionOnClientAndServerOnLevel1()
+      throws InterruptedException, IOException {
 
     doRequest(true, true, 1);
   }
 
   @Test
-  public void testRequestWithCompressionOnClientAndServerOnLevel6() throws InterruptedException, IOException {
+  public void testRequestWithCompressionOnClientAndServerOnLevel6()
+      throws InterruptedException, IOException {
 
     doRequest(true, true, 6);
   }
 
   @Test
-  public void testRequestWithCompressionOnClientAndServerOnLevel9() throws InterruptedException, IOException {
+  public void testRequestWithCompressionOnClientAndServerOnLevel9()
+      throws InterruptedException, IOException {
 
     doRequest(true, true, 9);
   }
 
-  @Test(expected=org.apache.avro.AvroRemoteException.class)
+  @Test(expected = org.apache.avro.AvroRemoteException.class)
   public void testRequestWithCompressionOnServerOnly() throws InterruptedException, IOException {
     //This will fail because both client and server need compression on
     doRequest(true, false, 6);
   }
 
-  @Test(expected=org.apache.avro.AvroRemoteException.class)
+  @Test(expected = org.apache.avro.AvroRemoteException.class)
   public void testRequestWithCompressionOnClientOnly() throws InterruptedException, IOException {
     //This will fail because both client and server need compression on
     doRequest(false, true, 6);
   }
 
-  private void doRequest(boolean serverEnableCompression, boolean clientEnableCompression, int compressionLevel) throws InterruptedException, IOException {
-    boolean bound = false;
+  private void doRequest(boolean serverEnableCompression, boolean clientEnableCompression,
+                         int compressionLevel) throws InterruptedException, IOException {
 
-    for (int i = 0; i < 100 && !bound; i++) {
-      try {
-        Context context = new Context();
-        context.put("port", String.valueOf(selectedPort = 41414 + i));
-        context.put("bind", "0.0.0.0");
-        context.put("threads", "50");
-        if (serverEnableCompression) {
-          context.put("compression-type", "deflate");
-        } else {
-          context.put("compression-type", "none");
-        }
-
-        Configurables.configure(source, context);
-
-        source.start();
-        bound = true;
-      } catch (ChannelException e) {
-        /*
-         * NB: This assume we're using the Netty server under the hood and the
-         * failure is to bind. Yucky.
-         */
-      }
+    Context context = new Context();
+    context.put("port", String.valueOf(selectedPort = getFreePort()));
+    context.put("bind", "0.0.0.0");
+    context.put("threads", "50");
+    if (serverEnableCompression) {
+      context.put("compression-type", "deflate");
+    } else {
+      context.put("compression-type", "none");
     }
 
+    Configurables.configure(source, context);
+
+    source.start();
+
     Assert
         .assertTrue("Reached start or error", LifecycleController.waitForOneOf(
             source, LifecycleState.START_OR_ERROR));
@@ -249,6 +299,12 @@ public class TestAvroSource {
         source.getLifecycleState());
   }
 
+  private static int getFreePort() throws IOException {
+    try (ServerSocket socket = new ServerSocket(0)) {
+      return socket.getLocalPort();
+    }
+  }
+
   private static class CompressionChannelFactory extends
       NioClientSocketChannelFactory {
     private int compressionLevel;
@@ -273,32 +329,45 @@ public class TestAvroSource {
   }
 
   @Test
-  public void testSslRequest() throws InterruptedException, IOException {
-    boolean bound = false;
+  public void testSslRequestWithComponentKeystore() throws InterruptedException, IOException {
 
-    for (int i = 0; i < 10 && !bound; i++) {
-      try {
-        Context context = new Context();
+    Context context = new Context();
 
-        context.put("port", String.valueOf(selectedPort = 41414 + i));
-        context.put("bind", "0.0.0.0");
-        context.put("ssl", "true");
-        context.put("keystore", "src/test/resources/server.p12");
-        context.put("keystore-password", "password");
-        context.put("keystore-type", "PKCS12");
+    context.put("port", String.valueOf(selectedPort = getFreePort()));
+    context.put("bind", "0.0.0.0");
+    context.put("ssl", "true");
+    context.put("keystore", "src/test/resources/server.p12");
+    context.put("keystore-password", "password");
+    context.put("keystore-type", "PKCS12");
 
-        Configurables.configure(source, context);
+    Configurables.configure(source, context);
 
-        source.start();
-        bound = true;
-      } catch (ChannelException e) {
-        /*
-         * NB: This assume we're using the Netty server under the hood and the
-         * failure is to bind. Yucky.
-         */
-        Thread.sleep(100);
-      }
-    }
+    doSslRequest();
+  }
+
+  @Test
+  public void testSslRequestWithGlobalKeystore() throws InterruptedException, IOException {
+
+    System.setProperty("javax.net.ssl.keyStore", "src/test/resources/server.p12");
+    System.setProperty("javax.net.ssl.keyStorePassword", "password");
+    System.setProperty("javax.net.ssl.keyStoreType", "PKCS12");
+
+    Context context = new Context();
+
+    context.put("port", String.valueOf(selectedPort = getFreePort()));
+    context.put("bind", "0.0.0.0");
+    context.put("ssl", "true");
+
+    Configurables.configure(source, context);
+
+    doSslRequest();
+
+    System.clearProperty("javax.net.ssl.keyStore");
+    System.clearProperty("javax.net.ssl.keyStorePassword");
+  }
+
+  private void doSslRequest() throws InterruptedException, IOException {
+    source.start();
 
     Assert
         .assertTrue("Reached start or error", LifecycleController.waitForOneOf(
@@ -308,7 +377,7 @@ public class TestAvroSource {
 
     AvroSourceProtocol client = SpecificRequestor.getClient(
         AvroSourceProtocol.class, new NettyTransceiver(new InetSocketAddress(
-        selectedPort), new SSLChannelFactory()));
+            selectedPort), new SSLChannelFactory()));
 
     AvroFlumeEvent avroEvent = new AvroFlumeEvent();
 
@@ -426,12 +495,20 @@ public class TestAvroSource {
     doIpFilterTest(localhost, "deny:ip:" +
         localhost.getHostAddress().substring(0, 3) + "*,allow:ip:*",
         false, false);
+
+    // Private lambda expression to check the received FlumeException within this test
+    Consumer<Exception> exceptionChecker = (Exception ex) -> {
+      logger.info("Received an expected exception", ex);
+      //covers all ipFilter related exceptions
+      Assert.assertTrue("Expected an ipFilterRules related exception",
+          ex.getMessage().contains("ipFilter"));
+    };
+
     try {
       doIpFilterTest(localhost, null, false, false);
-      Assert.fail(
-        "The null ipFilterRules config should have thrown an exception.");
+      Assert.fail("The null ipFilterRules config should have thrown an exception.");
     } catch (FlumeException e) {
-      //Do nothing
+      exceptionChecker.accept(e);
     }
 
     try {
@@ -439,57 +516,44 @@ public class TestAvroSource {
       Assert.fail("The empty string ipFilterRules config should have thrown "
           + "an exception");
     } catch (FlumeException e) {
-      //Do nothing
+      exceptionChecker.accept(e);
     }
 
     try {
       doIpFilterTest(localhost, "homer:ip:45.4.23.1", true, false);
       Assert.fail("Bad ipFilterRules config should have thrown an exception.");
     } catch (FlumeException e) {
-      //Do nothing
+      exceptionChecker.accept(e);
     }
+
     try {
       doIpFilterTest(localhost, "allow:sleeps:45.4.23.1", true, false);
       Assert.fail("Bad ipFilterRules config should have thrown an exception.");
     } catch (FlumeException e) {
-      //Do nothing
+      exceptionChecker.accept(e);
     }
   }
 
   public void doIpFilterTest(InetAddress dest, String ruleDefinition,
       boolean eventShouldBeAllowed, boolean testWithSSL)
       throws InterruptedException, IOException {
-    boolean bound = false;
-
-    for (int i = 0; i < 100 && !bound; i++) {
-      try {
-        Context context = new Context();
-        context.put("port", String.valueOf(selectedPort = 41414 + i));
-        context.put("bind", "0.0.0.0");
-        context.put("ipFilter", "true");
-        if (ruleDefinition != null) {
-          context.put("ipFilterRules", ruleDefinition);
-        }
-        if (testWithSSL) {
-          logger.info("Client testWithSSL" + testWithSSL);
-          context.put("ssl", "true");
-          context.put("keystore", "src/test/resources/server.p12");
-          context.put("keystore-password", "password");
-          context.put("keystore-type", "PKCS12");
-        }
-
-        Configurables.configure(source, context);
-
-        source.start();
-        bound = true;
-      } catch (ChannelException e) {
-        /*
-         * NB: This assume we're using the Netty server under the hood and the
-         * failure is to bind. Yucky.
-         */
-        Thread.sleep(100);
-      }
+    Context context = new Context();
+    context.put("port", String.valueOf(selectedPort = getFreePort()));
+    context.put("bind", "0.0.0.0");
+    context.put("ipFilter", "true");
+    if (ruleDefinition != null) {
+      context.put("ipFilterRules", ruleDefinition);
     }
+    if (testWithSSL) {
+      logger.info("Client testWithSSL" + testWithSSL);
+      context.put("ssl", "true");
+      context.put("keystore", "src/test/resources/server.p12");
+      context.put("keystore-password", "password");
+      context.put("keystore-type", "PKCS12");
+    }
+    // Invalid configuration may result in a FlumeException
+    Configurables.configure(source, context);
+    source.start();
 
     Assert
         .assertTrue("Reached start or error", LifecycleController.waitForOneOf(
@@ -502,15 +566,15 @@ public class TestAvroSource {
     try {
       if (testWithSSL) {
         nettyTransceiver = new NettyTransceiver(
-          new InetSocketAddress (dest, selectedPort),
-          new SSLChannelFactory());
+            new InetSocketAddress(dest, selectedPort),
+            new SSLChannelFactory());
         client = SpecificRequestor.getClient(
-          AvroSourceProtocol.class, nettyTransceiver);
+            AvroSourceProtocol.class, nettyTransceiver);
       } else {
         nettyTransceiver = new NettyTransceiver(
-          new InetSocketAddress (dest, selectedPort));
+            new InetSocketAddress(dest, selectedPort));
         client = SpecificRequestor.getClient(
-          AvroSourceProtocol.class, nettyTransceiver);
+            AvroSourceProtocol.class, nettyTransceiver);
       }
 
       AvroFlumeEvent avroEvent = new AvroFlumeEvent();
@@ -523,7 +587,7 @@ public class TestAvroSource {
       Assert.assertEquals(Status.OK, status);
     } catch (IOException e) {
       Assert.assertTrue("Should have been allowed: " + ruleDefinition,
-        !eventShouldBeAllowed);
+          !eventShouldBeAllowed);
       return;
     } finally {
       if (nettyTransceiver != null) {
@@ -550,4 +614,26 @@ public class TestAvroSource {
     Assert.assertEquals("Server is stopped", LifecycleState.STOP,
         source.getLifecycleState());
   }
+
+  @Test
+  public void testErrorCounterChannelWriteFail() throws Exception {
+    Context context = new Context();
+    context.put("port", String.valueOf(selectedPort = getFreePort()));
+    context.put("bind", "0.0.0.0");
+    source.configure(context);
+    ChannelProcessor cp = Mockito.mock(ChannelProcessor.class);
+    doThrow(new ChannelException("dummy")).when(cp).processEvent(any(Event.class));
+    doThrow(new ChannelException("dummy")).when(cp).processEventBatch(anyListOf(Event.class));
+    source.setChannelProcessor(cp);
+    source.start();
+    AvroFlumeEvent avroEvent = new AvroFlumeEvent();
+    avroEvent.setHeaders(new HashMap<CharSequence, CharSequence>());
+    avroEvent.setBody(ByteBuffer.wrap("Hello avro ssl".getBytes()));
+    source.append(avroEvent);
+    source.appendBatch(Arrays.asList(avroEvent));
+    SourceCounter sc = (SourceCounter) Whitebox.getInternalState(source, "sourceCounter");
+    Assert.assertEquals(2, sc.getChannelWriteFail());
+    source.stop();
+  }
+
 }
