@@ -19,6 +19,7 @@ package org.apache.flume.sink.kite;
 
 import org.apache.flume.auth.FlumeAuthenticationUtil;
 import org.apache.flume.auth.PrivilegedExecutor;
+import org.apache.flume.conf.BatchSizeSupported;
 import org.apache.flume.sink.kite.parser.EntityParserFactory;
 import org.apache.flume.sink.kite.parser.EntityParser;
 import org.apache.flume.sink.kite.policy.FailurePolicy;
@@ -69,7 +70,7 @@ import org.kitesdk.data.Formats;
  * and loading a Dataset by name, {@code kite.dataset.name}, and namespace,
  * {@code kite.dataset.namespace}.
  */
-public class DatasetSink extends AbstractSink implements Configurable {
+public class DatasetSink extends AbstractSink implements Configurable, BatchSizeSupported {
 
   private static final Logger LOG = LoggerFactory.getLogger(DatasetSink.class);
 
@@ -104,12 +105,12 @@ public class DatasetSink extends AbstractSink implements Configurable {
   /**
    * The last time the writer rolled.
    */
-  private long lastRolledMillis = 0l;
+  private long lastRolledMillis = 0L;
 
   /**
    * The raw number of bytes parsed.
    */
-  private long bytesParsed = 0l;
+  private long bytesParsed = 0L;
 
   /**
    * A class for parsing Kite entities from Flume Events.
@@ -225,7 +226,7 @@ public class DatasetSink extends AbstractSink implements Configurable {
    */
   @VisibleForTesting
   void roll() {
-    this.lastRolledMillis = 0l;
+    this.lastRolledMillis = 0L;
   }
 
   @VisibleForTesting
@@ -391,7 +392,7 @@ public class DatasetSink extends AbstractSink implements Configurable {
    */
   @VisibleForTesting
   void createWriter() throws EventDeliveryException {
-    // reset the commited flag whenver a new writer is created
+    // reset the commited flag whenever a new writer is created
     committedBatch = false;
     try {
       View<GenericRecord> view;
@@ -434,7 +435,7 @@ public class DatasetSink extends AbstractSink implements Configurable {
 
       // Reset the last rolled time and the metrics
       this.lastRolledMillis = System.currentTimeMillis();
-      this.bytesParsed = 0l;
+      this.bytesParsed = 0L;
     } catch (DatasetNotFoundException ex) {
       throw new EventDeliveryException("Dataset " + datasetUri + " not found."
           + " The dataset must be created before Flume can write to it.", ex);
@@ -558,7 +559,7 @@ public class DatasetSink extends AbstractSink implements Configurable {
       try {
         // If the transaction wasn't committed before we got the exception, we
         // need to rollback.
-          transaction.rollback();
+        transaction.rollback();
       } catch (RuntimeException ex) {
         LOG.error("Transaction rollback failed: " + ex.getLocalizedMessage());
         LOG.debug("Exception follows.", ex);
@@ -567,7 +568,7 @@ public class DatasetSink extends AbstractSink implements Configurable {
         this.transaction = null;
       }
     }
-}
+  }
 
   /**
    * Get the name of the dataset from the URI
@@ -579,4 +580,9 @@ public class DatasetSink extends AbstractSink implements Configurable {
     return Registration.lookupDatasetUri(URI.create(
         uri.getRawSchemeSpecificPart())).second().get("dataset");
   }
+
+  @Override
+  public long getBatchSize() {
+    return batchSize;
+  }
 }
